--[[

This program creates a status page for my GL-INET AR150 router. It is part of a
configuration setup described at:

https://hoevenstein.nl/my-openwrt-tor-configuration

Copyright (C) 2016 Rob van der Hoeven

License.
--------

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

--]]

cgi_200_response = [[
Status: 200 OK
Content-Type: text/html
Server: Torbox Status 0.5

]]

html_start = [[
<!DOCTYPE html>
<html lang="en">
    <head>
        <title>Torbox status</title>
        <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
        <meta name="description" content="current status">
        <link rel="stylesheet" type="text/css" href="torbox.css" >
    </head>
    <body>
        <header class="top">
            <h1><a href="torbox">Torbox</a></h1>
            <nav>
                <a class="active" href="torbox">Status</a>
                <a href="/">Settings</a>
            </nav>
        </header>
]]

html_end = [[
        <footer>
            <small><strong>This page is 100%</strong>  Google Analytics free!   Cookie free!   Javascript free!   Web bug free!   Valid HTML and CSS!  <strong>Respecting your privacy!</strong></small>
        </footer>
    </body>
</html>
]]

html_interfaces_start = [[
<section >
    <h3>Interfaces</h3>
]]

html_interfaces_end = [[
    <div style="clear:both;"></div>
</section>
]]

html_wired_interface_stats = [[
<dl>
  <dt>%s</dt>
  <dd><span>Address:</span>%s</dd>
  <dd><span>Netmask:</span>%s</dd>
  <dd><span>MAC:</span>%s</dd>
  <dd><span>Received:</span>%s</dd>
  <dd><span>Transmitted:</span>%s</dd>
</dl>
]]

html_wireless_interface_stats = [[
<dl>
  <dt>%s</dt>
  <dd><span>SSID:</span>%s</dd>
  <dd><span>Address:</span>%s</dd>
  <dd><span>Netmask:</span>%s</dd>
  <dd><span>MAC:</span>%s</dd>
  <dd><span>Received:</span>%s</dd>
  <dd><span>Transmitted:</span>%s</dd>
</dl>
]]

html_memory_start = [[
<section >
    <h3>Memory</h3>
]]

html_memory_end = html_interfaces_end

html_ram_stats = [[
<dl>
  <dt>RAM (KB)</dt>
  <dd><span>Total:</span>%s</dd>
  <dd><span>Used:</span>%s</dd>
  <dd><span>Free:</span>%s</dd>
  <dd><span>Buffers:</span>%s</dd>
  <dd><span>+/- Buffers</span>&nbsp;</dd>
  <dd><span>Used:</span>%s</dd>
  <dd><span>Free:</span>%s</dd>
</dl>
<dl>
  <dt>RAM (%%)</dt>
  <dd class="pie">%s<span>%s%% free</span></dd>
</dl>
]]

html_filesystem_stats = [[
<dl>
  <dt>%s</dt>
  <dd><span>Mountpoint:</span>%s</dd>
  <dd><span>Size:</span>%s</dd>
  <dd><span>Used:</span>%s</dd>
  <dd><span>Free:</span>%s</dd>
</dl>
<dl>
  <dt>%s (%%)</dt>
  <dd class="pie">%s<span>%s%% free</span></dd>
</dl>
]]

html_svg_pie = [[
<svg width="150px" height="150px">
    <circle cx="75" cy="75" r="50" stroke="red" fill="green" stroke-width="0"/>
    <path d="M75 75 L 75 25 A 50 50 0 %s %s %s %s L 75 75 Z" stroke="black" fill="gray" stroke-width="2" fill-opacity="1"/>
</svg>
]]

function read_interfaces()
    local interfaces = {}
    local properties = {}
    local f = io.popen("/sbin/ifconfig")

    for line in f:lines("*all") do
        if #line > 0 and string.sub(line,1,1) ~= " " then
            properties={}
            local interface=string.match(line,"([^%s]+)")
            interfaces[interface] = properties

            properties["mac_address"]    = string.match(line,"(%x%x:%x%x:%x%x:%x%x:%x%x:%x%x)") or ""
        elseif string.find(line,"inet addr") then
            properties["network_address"]=string.match(line, "inet addr:(%d+%.%d+%.%d+%.%d+)")
            properties["network_mask"]   =string.match(line, "Mask:(%d+%.%d+%.%d+%.%d+)")
        elseif string.find(line, "RX bytes:") then
            properties["rx_bytes"]       =string.match(line, "RX bytes:(%d+)")
            properties["tx_bytes"]       =string.match(line, "TX bytes:(%d+)")
        end
    end

    f:close()
    
    -- get SSID's
    
    local f = io.popen("iwinfo")

    for line in f:lines("*all") do
        if string.sub(line,1,4) == "wlan" then
            interface, ssid = string.match(line,"([^%s]+)%s*ESSID: [\"](.-)[\"]");
            interfaces[interface]["ssid"] = ssid
        end
    end

    f:close()

    return interfaces
end

function read_filesystems()
    local filesystem_property_names = {"Size","Used","Available","Used%","Mountpoint"}
    local filesystems = {}
    local f = io.popen("/bin/df -h")

    for line in f:lines("*all") do
        if string.find(line,"rootfs") or string.find(line,"/dev/sda") then
            local properties={}
            filesystems[string.match(line, "([^%s]+)%s+")] = properties

            local i=1
            for value in string.gmatch(line, "%s+([^%s]+)") do
                properties[filesystem_property_names[i]]=value
                i=i+1
            end
        end
    end

    f:close()

    return filesystems
end

function read_ram()
    local memory = {}
    local f = io.popen("/usr/bin/free -k")

    for line in f:lines("*all") do
        if string.find(line,"Mem:") then
            memory["total"], memory["used"], memory["free"], memory["shared"], memory["buffers"] =
                string.match(line,"(%d+)%s+(%d+)%s+(%d+)%s+(%d+)%s+(%d+)")
        elseif string.find(line, "Swap:") then
            memory["swap_total"], memory["swap_used"], memory["swap_free"] =
                string.match(line,"(%d+)%s+(%d+)%s+(%d+)")
        end
    end

    f:close()

    return memory
end;

--[[  not used now, maybe needed in future
function read_configuration(configuration_type, section_type)
    local configuration = {}
    local cursor = uci.cursor()

    cursor:foreach(configuration_type, section_type, function(section)
        local name = section[".name"]
        local properties = {}

        configuration[name] = properties

        for name, value in pairs(section) do
            if string.sub(name,1,1) ~= "." then properties[name] = value end
        end
    end)

    return configuration
end
--]]

function html_interfaces()
    local interfaces = read_interfaces()
    local interface_names ={}

    for interface_name in pairs(interfaces) do
        if interface_name ~= "lo" then
            interface_names[#interface_names+1]=interface_name
        end
    end

    table.sort(interface_names)

    local html=html_interfaces_start

    for i, interface_name in ipairs(interface_names) do
        local properties=interfaces[interface_name]

        if properties.ssid then
            html= html .. string.format(html_wireless_interface_stats, interface_name, properties.ssid,
                properties.network_address or "part of bridge", properties.network_mask or "part of bridge", properties.mac_address,
                properties.rx_bytes, properties.tx_bytes)
        else
            html= html .. string.format(html_wired_interface_stats, interface_name,
                properties.network_address or "part of bridge", properties.network_mask or "part of bridge", properties.mac_address,
                properties.rx_bytes, properties.tx_bytes)
        end
    end

    return html .. html_interfaces_end
end

function svg_pie(fraction)
    local r, cx, cy = 50, 75, 75
    local angle = fraction * 360
    local x = r * math.sin(math.rad(angle))
    local y = r * math.cos(math.rad(angle))
    local large_arc = (angle > 180) and  1 or 0
    local sweep = 1

    return string.format(html_svg_pie, large_arc, sweep, x+cx, -y+cy)
end

function html_filesystems()
    local filesystems = read_filesystems()
    local html = ""

    for filesystem, properties in pairs(filesystems) do
        local fraction=tonumber(string.match(properties["Used%"], "%d+")) / 100.0
        html = html .. string.format(html_filesystem_stats, filesystem,
            properties.Mountpoint,
            properties.Size,
            properties.Used,
            properties.Available,
            filesystem,
            svg_pie(fraction),
            100 - tonumber(string.match(properties["Used%"],"%d+")))
    end

    return html
end

function html_ram()
    local ram = read_ram()
    local fraction=(ram.used - ram.buffers)/ram.total

    return string.format(html_ram_stats,
        ram.total, ram.used, ram.free, ram.buffers,
        ram.used-ram.buffers, ram.free+ram.buffers,
        svg_pie(fraction), string.format("%.0f", 100 - 100*fraction))
end

function handle_request(env)
    local output = cgi_200_response .. html_start
    output = output .. html_memory_start .. html_ram() .. html_filesystems() .. html_memory_end
    output = output .. html_interfaces() .. html_end

    uhttpd.send(output)
end

function handle_request_test()
    local env=""
    uhttpd={}

    local f=io.open("/www/torboxtest.html", "w")

    function send(output)
        f:write(output)
    end

    uhttpd.send=send
    handle_request(env)

    f:close()
end

function read_ram_test()
    local ram = read_ram()

    print(" total        : ", ram.total)
    print(" used         : ", ram.used)
    print(" used-buffers : ", ram.used-ram.buffers)
    print(" free         : ", ram.free)
    print(" free+buffers : ", ram.free+ram.buffers)
    print(" shared       : ", ram.shared)
    print(" buffers      : ", ram.buffers)
    print(" swap_total   : ", ram.swap_total)
    print(" swap_used    : ", ram.swap_used)
    print(" swap_free    : ", ram.swap_free)
end

function read_networks_test()
    local networks=read_configuration("network", "interface")

    for network, properties in pairs(networks) do
        print("Network : " .. network)
        print("================================")

        for name, value in pairs(properties) do
            print("    " .. name .. " = " .. tostring(value))
        end
    end
end

-- read_networks_test()
-- handle_request_test()
-- read_ram_test()
